#include <stdarg.h>
#include <stdio.h>

#include "mk20dx128.h"

#define _BV(n) (1<<(n))
void i2c0_isr(void);
void porta_isr(void);
void i2c_sda_rising_isr(void);
uint8_t i2cSDAint = 0;

int main(void) {

	// Onboard LED (Tsy 13)
	PORTC_PCR5  |= _BV(8); 														  // Digital IO PTC5
	GPIOC_PDDR |= _BV(5);																   // Output
	GPIOC_PDOR |= _BV(5);																	 // High

		///////////////////////// I2C


	SIM_SCGC4 |= SIM_SCGC4_I2C0;												 // Enable I2C Clock
	//I2C0_C2 = 0x00;
	I2C0_C2 = I2C_C2_HDRS;														   // High Drive
	I2C0_A1	= 0x44<<1;																		  // Address
	I2C0_RA = 0x0;											 // Range Address Null = > Only One Addr
	// We'll use PTB0, PTB1 for I2C Slave (Tsy 16 - 17)
	PORTB_PCR0 = PORT_PCR_MUX(2)|PORT_PCR_ODE|PORT_PCR_SRE|PORT_PCR_DSE; 																 // I2C0_SCL
	PORTB_PCR1 = PORT_PCR_MUX(2)|PORT_PCR_ODE|PORT_PCR_SRE|PORT_PCR_DSE; 																 // I2C0_SDA
	I2C0_F = 0x85;																//
	I2C0_FLT = 0x4;																  // Filter Glitches
	I2C0_C1 = I2C_C1_IICEN|I2C_C1_IICIE;
		///////////////////////// Run !
	NVIC_ENABLE_IRQ(IRQ_PORTC);
	NVIC_ENABLE_IRQ(IRQ_I2C0);

	uint32_t n;
	while (1) {
		n=0;
		while(n++<0xFFFFF);
		GPIOC_PTOR = _BV(5);

	}
}


void i2c0_isr(void) {
	uint8_t i2cS,i2cC1,c;

	i2cS = I2C0_S;
	i2cC1 = I2C0_C1;
	if(!(i2cC1 & I2C_C1_MST)) {
		if(i2cS & I2C_S_ARBL) {
				// Arbitration Lost
			I2C0_S = I2C_S_ARBL;    // clear arbl flag
			if(!(i2cS & I2C_S_IAAS)) {
				I2C0_S = I2C_S_IICIF; // clear intr
				return;
			}
		}
		if(i2cS & I2C_S_IAAS) {
			// If in Slave Rx already, then RepSTART occured, run callback
			// Is Addressed As Slave
			if(i2cS & I2C_S_SRW) {
				// Begin Slave Transmit
				I2C0_C1 = I2C_C1_IICEN | I2C_C1_IICIE;
				c = I2C0_D>>1;
				//i2c_t3::rxAddr = (I2C0_D>>1); // read to get target addr
				I2C0_C1 = I2C_C1_IICEN | I2C_C1_IICIE | I2C_C1_TX;
				I2C0_D = 0x42;
			} else {
				// Begin Slave Receive
				PORTB_PCR1 |= PORT_PCR_IRQC(6);
				i2cSDAint = 1;
				//attachInterrupt(17, i2c_t3::sda_rising_isr, RISING);
				I2C0_C1 = I2C_C1_IICEN | I2C_C1_IICIE;
				c = I2C0_D >> 1;
				//i2c_t3::rxAddr = (I2C0_D>>1); // read to get target addr
			}
			I2C0_S = I2C_S_IICIF; // clear intr
			return;
		}
		if(i2cC1 & I2C_C1_TX) {
				// Continue Slave Transmit
			if((i2cS & I2C_S_RXAK) == 0) {
				// Master ACK'd previous byte
				I2C0_D = 0x43;
				I2C0_C1 = I2C_C1_IICEN | I2C_C1_IICIE | I2C_C1_TX;
			} else {
				// Master did not ACK previous byte
				I2C0_C1 = I2C_C1_IICEN | I2C_C1_IICIE; // switch to Rx mode
				//data = I2C0_D; // dummy read
				c = I2C0_D;
			}
		} else {
			// Continue Slave Receive
			PORTB_PCR1 |= PORT_PCR_IRQC(6);
			i2cSDAint = 1;
			//attachInterrupt(17, i2c_t3::sda_rising_isr, RISING);
			//data = I2C0_D;
			c = I2C0_D;
		}
	}
	I2C0_S = I2C_S_IICIF;
}

void portb_isr(void) {
	uint32_t isfr = PORTB_ISFR;
	PORTB_ISFR = isfr;
	if((isfr & _BV(1)) && i2cSDAint)
		i2c_sda_rising_isr();
}

void i2c_sda_rising_isr(void) {
    uint8_t i2cS = I2C0_S; // capture status first, can change if ISR is too slow

    if(!(i2cS & I2C_S_BUSY)) {
		PORTB_PCR1 |= PORT_PCR_IRQC(0);
		i2cSDAint = 0;
		//detachInterrupt(17);
        /*if(user_onReceive != NULL)
        {
            rxBufferIndex = 0;
            user_onReceive(rxBufferLength);
        }*/
    }
}